/* Window.c generated by valac 0.56.8, the Vala compiler
 * generated from Window.vala, do not modify */

/**
 * SPDX-FileCopyrightText: Copyright © 2020-2024 Louis Brauer <louis@brauer.family>
 * SPDX-FileCopyrightText: Copyright © 2024 technosf <https://github.com/technosf>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @file Window.vala
 *
 * @brief Defines the main application window for the Tuner application.
 *
 * This file contains the Window class, which is responsible for creating and
 * managing the main application window. It handles the major layout, user interface
 * elements, and interactions with other non-display components of the application.
 *
 * The Window class inherits from Gtk.ApplicationWindow and implements various
 * features such as a header bar, main display and player controls.
 * It also manages application settings and handles user actions like playback
 * control, station selection, and theme adjustments.
 *
 * @see Tuner.Application
 * @see Tuner.PlayerController
 * @see Tuner.DirectoryController
 * @see Tuner.HeaderBar
 * @see Tuner.Display
 */

#include <gtk/gtk.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <gee.h>
#include <gdk/gdk.h>

#define TUNER_WINDOW_WINDOW_NAME "Tuner"
#define TUNER_WINDOW_ACTION_PREFIX "win."
#define TUNER_WINDOW_ACTION_PAUSE "action_pause"
#define TUNER_WINDOW_ACTION_QUIT "action_quit"
#define TUNER_WINDOW_ACTION_HIDE "action_hide"
#define TUNER_WINDOW_ACTION_ABOUT "action_about"
#define TUNER_WINDOW_ACTION_DISABLE_TRACKING "action_disable_tracking"
#define TUNER_WINDOW_ACTION_ENABLE_AUTOPLAY "action_enable_autoplay"
#define TUNER_WINDOW_ACTION_START_ON_STARRED "action_starred_start"
#define TUNER_WINDOW_ACTION_STREAM_INFO "action_stream_info"
#define TUNER_WINDOW_ACTION_STREAM_INFO_FAST "action_stream_info_fast"
#define TUNER_WINDOW_CSS "io/github/louis77/tuner/Application.css"
#define TUNER_WINDOW_NOTIFICATION_PLAYING_BACKGROUND _ ("Playing in background")
#define TUNER_WINDOW_NOTIFICATION_CLICK_RESUME _ ("Click here to resume window. To quit Tuner, pause playback and close t" \
"he window.")
#define TUNER_WINDOW_NOTIFICATION_APP_RESUME_WINDOW "app.resume-window"
#define TUNER_WINDOW_NOTIFICATION_APP_PLAYING_CONTINUE "continue-playing"
#define TUNER_WINDOW_RANDOM_CATEGORIES 5
#define TUNER_WINDOW_GEOMETRY_MIN_HEIGHT 440
#define TUNER_WINDOW_GEOMETRY_MIN_WIDTH 600
#define TUNER_APPLICATION_APP_ID "com.github.louis77.tuner"
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define TUNER_TYPE_WINDOW (tuner_window_get_type ())
#define TUNER_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_WINDOW, TunerWindow))
#define TUNER_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_WINDOW, TunerWindowClass))
#define TUNER_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_WINDOW))
#define TUNER_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_WINDOW))
#define TUNER_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_WINDOW, TunerWindowClass))

typedef struct _TunerWindow TunerWindow;
typedef struct _TunerWindowClass TunerWindowClass;
typedef struct _TunerWindowPrivate TunerWindowPrivate;

#define TUNER_TYPE_SETTINGS (tuner_settings_get_type ())
#define TUNER_SETTINGS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_SETTINGS, TunerSettings))
#define TUNER_SETTINGS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_SETTINGS, TunerSettingsClass))
#define TUNER_IS_SETTINGS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_SETTINGS))
#define TUNER_IS_SETTINGS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_SETTINGS))
#define TUNER_SETTINGS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_SETTINGS, TunerSettingsClass))

typedef struct _TunerSettings TunerSettings;
typedef struct _TunerSettingsClass TunerSettingsClass;

#define TUNER_TYPE_PLAYER_CONTROLLER (tuner_player_controller_get_type ())
#define TUNER_PLAYER_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_PLAYER_CONTROLLER, TunerPlayerController))
#define TUNER_PLAYER_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_PLAYER_CONTROLLER, TunerPlayerControllerClass))
#define TUNER_IS_PLAYER_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_PLAYER_CONTROLLER))
#define TUNER_IS_PLAYER_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_PLAYER_CONTROLLER))
#define TUNER_PLAYER_CONTROLLER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_PLAYER_CONTROLLER, TunerPlayerControllerClass))

typedef struct _TunerPlayerController TunerPlayerController;
typedef struct _TunerPlayerControllerClass TunerPlayerControllerClass;

#define TUNER_TYPE_DIRECTORY_CONTROLLER (tuner_directory_controller_get_type ())
#define TUNER_DIRECTORY_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_DIRECTORY_CONTROLLER, TunerDirectoryController))
#define TUNER_DIRECTORY_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_DIRECTORY_CONTROLLER, TunerDirectoryControllerClass))
#define TUNER_IS_DIRECTORY_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_DIRECTORY_CONTROLLER))
#define TUNER_IS_DIRECTORY_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_DIRECTORY_CONTROLLER))
#define TUNER_DIRECTORY_CONTROLLER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_DIRECTORY_CONTROLLER, TunerDirectoryControllerClass))

typedef struct _TunerDirectoryController TunerDirectoryController;
typedef struct _TunerDirectoryControllerClass TunerDirectoryControllerClass;

#define TUNER_TYPE_HEADER_BAR (tuner_header_bar_get_type ())
#define TUNER_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_HEADER_BAR, TunerHeaderBar))
#define TUNER_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_HEADER_BAR, TunerHeaderBarClass))
#define TUNER_IS_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_HEADER_BAR))
#define TUNER_IS_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_HEADER_BAR))
#define TUNER_HEADER_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_HEADER_BAR, TunerHeaderBarClass))

typedef struct _TunerHeaderBar TunerHeaderBar;
typedef struct _TunerHeaderBarClass TunerHeaderBarClass;

#define TUNER_TYPE_DISPLAY (tuner_display_get_type ())
#define TUNER_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_DISPLAY, TunerDisplay))
#define TUNER_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_DISPLAY, TunerDisplayClass))
#define TUNER_IS_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_DISPLAY))
#define TUNER_IS_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_DISPLAY))
#define TUNER_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_DISPLAY, TunerDisplayClass))

typedef struct _TunerDisplay TunerDisplay;
typedef struct _TunerDisplayClass TunerDisplayClass;
enum  {
	TUNER_WINDOW_0_PROPERTY,
	TUNER_WINDOW_SETTINGS_PROPERTY,
	TUNER_WINDOW_PLAYER_CTRL_PROPERTY,
	TUNER_WINDOW_DIRECTORY_PROPERTY,
	TUNER_WINDOW_ACTIVE_PROPERTY,
	TUNER_WINDOW_WIDTH_PROPERTY,
	TUNER_WINDOW_HEIGHT_PROPERTY,
	TUNER_WINDOW_NUM_PROPERTIES
};
static GParamSpec* tuner_window_properties[TUNER_WINDOW_NUM_PROPERTIES];
typedef void (*GSimpleActionActivateCallback) (GSimpleAction* action, GVariant* parameter, gpointer user_data);
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define TUNER_TYPE_APPLICATION (tuner_application_get_type ())
#define TUNER_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_APPLICATION, TunerApplication))
#define TUNER_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_APPLICATION, TunerApplicationClass))
#define TUNER_IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_APPLICATION))
#define TUNER_IS_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_APPLICATION))
#define TUNER_APPLICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_APPLICATION, TunerApplicationClass))

typedef struct _TunerApplication TunerApplication;
typedef struct _TunerApplicationClass TunerApplicationClass;

#define TUNER_MODEL_TYPE_FAVICON (tuner_model_favicon_get_type ())
#define TUNER_MODEL_FAVICON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_MODEL_TYPE_FAVICON, TunerModelFavicon))
#define TUNER_MODEL_FAVICON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_MODEL_TYPE_FAVICON, TunerModelFaviconClass))
#define TUNER_MODEL_IS_FAVICON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_MODEL_TYPE_FAVICON))
#define TUNER_MODEL_IS_FAVICON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_MODEL_TYPE_FAVICON))
#define TUNER_MODEL_FAVICON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_MODEL_TYPE_FAVICON, TunerModelFaviconClass))

typedef struct _TunerModelFavicon TunerModelFavicon;
typedef struct _TunerModelFaviconClass TunerModelFaviconClass;

#define TUNER_MODEL_TYPE_STATION (tuner_model_station_get_type ())
#define TUNER_MODEL_STATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_MODEL_TYPE_STATION, TunerModelStation))
#define TUNER_MODEL_STATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_MODEL_TYPE_STATION, TunerModelStationClass))
#define TUNER_MODEL_IS_STATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_MODEL_TYPE_STATION))
#define TUNER_MODEL_IS_STATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_MODEL_TYPE_STATION))
#define TUNER_MODEL_STATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_MODEL_TYPE_STATION, TunerModelStationClass))

typedef struct _TunerModelStation TunerModelStation;
typedef struct _TunerModelStationClass TunerModelStationClass;

#define TUNER_TYPE_STATION_SET (tuner_station_set_get_type ())
#define TUNER_STATION_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_STATION_SET, TunerStationSet))
#define TUNER_STATION_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_STATION_SET, TunerStationSetClass))
#define TUNER_IS_STATION_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_STATION_SET))
#define TUNER_IS_STATION_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_STATION_SET))
#define TUNER_STATION_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_STATION_SET, TunerStationSetClass))

typedef struct _TunerStationSet TunerStationSet;
typedef struct _TunerStationSetClass TunerStationSetClass;

#define TYPE_SOURCE_ERROR (source_error_get_type ())

#define TUNER_TYPE_ABOUT_DIALOG (tuner_about_dialog_get_type ())
#define TUNER_ABOUT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNER_TYPE_ABOUT_DIALOG, TunerAboutDialog))
#define TUNER_ABOUT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNER_TYPE_ABOUT_DIALOG, TunerAboutDialogClass))
#define TUNER_IS_ABOUT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNER_TYPE_ABOUT_DIALOG))
#define TUNER_IS_ABOUT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNER_TYPE_ABOUT_DIALOG))
#define TUNER_ABOUT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNER_TYPE_ABOUT_DIALOG, TunerAboutDialogClass))

typedef struct _TunerAboutDialog TunerAboutDialog;
typedef struct _TunerAboutDialogClass TunerAboutDialogClass;
#define _g_variant_unref0(var) ((var == NULL) ? NULL : (var = (g_variant_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
typedef enum  {
	TUNER_PLAYER_CONTROLLER_IS_BUFFERING,
	TUNER_PLAYER_CONTROLLER_IS_PAUSED,
	TUNER_PLAYER_CONTROLLER_IS_PLAYING,
	TUNER_PLAYER_CONTROLLER_IS_STOPPED,
	TUNER_PLAYER_CONTROLLER_IS_STOPPED_ERROR
} TunerPlayerControllerIs;

#define TUNER_PLAYER_CONTROLLER_TYPE_IS (tuner_player_controller_is_get_type ())
enum  {
	TUNER_WINDOW_REFRESH_SAVED_SEARCHES_SIG_SIGNAL,
	TUNER_WINDOW_NUM_SIGNALS
};
static guint tuner_window_signals[TUNER_WINDOW_NUM_SIGNALS] = {0};

struct _TunerWindow {
	GtkApplicationWindow parent_instance;
	TunerWindowPrivate * priv;
};

struct _TunerWindowClass {
	GtkApplicationWindowClass parent_class;
};

struct _TunerWindowPrivate {
	TunerSettings* _settings;
	TunerPlayerController* _player_ctrl;
	TunerDirectoryController* _directory;
	gboolean _active;
	gint _width;
	gint _height;
	TunerHeaderBar* _headerbar;
	TunerDisplay* _display;
	gboolean _start_on_starred;
};

typedef enum  {
	SOURCE_ERROR_UNAVAILABLE
} SourceError;
#define SOURCE_ERROR source_error_quark ()

static gint TunerWindow_private_offset;
static gpointer tuner_window_parent_class = NULL;

VALA_EXTERN GType tuner_window_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerWindow, g_object_unref)
VALA_EXTERN GType tuner_settings_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerSettings, g_object_unref)
VALA_EXTERN GType tuner_player_controller_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerPlayerController, g_object_unref)
VALA_EXTERN GType tuner_directory_controller_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerDirectoryController, g_object_unref)
VALA_EXTERN GType tuner_header_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerHeaderBar, g_object_unref)
VALA_EXTERN GType tuner_display_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerDisplay, g_object_unref)
VALA_EXTERN void tuner_window_on_toggle_playback (TunerWindow* self);
static void _tuner_window_on_toggle_playback_gsimple_action_activate_callback (GSimpleAction* action,
                                                                        GVariant* parameter,
                                                                        gpointer self);
static void tuner_window_on_action_quit (TunerWindow* self);
static void _tuner_window_on_action_quit_gsimple_action_activate_callback (GSimpleAction* action,
                                                                    GVariant* parameter,
                                                                    gpointer self);
static void tuner_window_on_action_about (TunerWindow* self);
static void _tuner_window_on_action_about_gsimple_action_activate_callback (GSimpleAction* action,
                                                                     GVariant* parameter,
                                                                     gpointer self);
VALA_EXTERN void tuner_window_on_action_disable_tracking (TunerWindow* self,
                                              GSimpleAction* action,
                                              GVariant* parameter);
static void _tuner_window_on_action_disable_tracking_gsimple_action_activate_callback (GSimpleAction* action,
                                                                                GVariant* parameter,
                                                                                gpointer self);
VALA_EXTERN void tuner_window_on_action_enable_autoplay (TunerWindow* self,
                                             GSimpleAction* action,
                                             GVariant* parameter);
static void _tuner_window_on_action_enable_autoplay_gsimple_action_activate_callback (GSimpleAction* action,
                                                                               GVariant* parameter,
                                                                               gpointer self);
VALA_EXTERN void tuner_window_on_action_start_on_starred (TunerWindow* self,
                                              GSimpleAction* action,
                                              GVariant* parameter);
static void _tuner_window_on_action_start_on_starred_gsimple_action_activate_callback (GSimpleAction* action,
                                                                                GVariant* parameter,
                                                                                gpointer self);
VALA_EXTERN void tuner_window_on_action_stream_info (TunerWindow* self,
                                         GSimpleAction* action,
                                         GVariant* parameter);
static void _tuner_window_on_action_stream_info_gsimple_action_activate_callback (GSimpleAction* action,
                                                                           GVariant* parameter,
                                                                           gpointer self);
VALA_EXTERN void tuner_window_on_action_stream_info_fast (TunerWindow* self,
                                              GSimpleAction* action,
                                              GVariant* parameter);
static void _tuner_window_on_action_stream_info_fast_gsimple_action_activate_callback (GSimpleAction* action,
                                                                                GVariant* parameter,
                                                                                gpointer self);
VALA_EXTERN GType tuner_application_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerApplication, g_object_unref)
VALA_EXTERN TunerWindow* tuner_window_new (TunerApplication* app,
                               TunerPlayerController* player,
                               TunerSettings* settings,
                               TunerDirectoryController* directory);
VALA_EXTERN TunerWindow* tuner_window_construct (GType object_type,
                                     TunerApplication* app,
                                     TunerPlayerController* player,
                                     TunerSettings* settings,
                                     TunerDirectoryController* directory);
static void tuner_window_add_widgets (TunerWindow* self);
static void tuner_window_check_online_status (TunerWindow* self);
VALA_EXTERN gboolean tuner_settings_get_start_on_starred (TunerSettings* self);
VALA_EXTERN void tuner_window_choose_starred_stations (TunerWindow* self);
VALA_EXTERN void tuner_display_choose_starred_stations (TunerDisplay* self);
VALA_EXTERN TunerHeaderBar* tuner_header_bar_new (TunerWindow* window);
VALA_EXTERN TunerHeaderBar* tuner_header_bar_construct (GType object_type,
                                            TunerWindow* window);
static void __lambda86_ (TunerWindow* self);
static void ___lambda86__tuner_header_bar_search_has_focus_sig (TunerHeaderBar* _sender,
                                                         gpointer self);
static void __lambda87_ (TunerWindow* self,
                  const gchar* text);
static void ___lambda87__tuner_header_bar_searching_for_sig (TunerHeaderBar* _sender,
                                                      const gchar* text,
                                                      gpointer self);
VALA_EXTERN TunerDirectoryController* tuner_window_get_directory (TunerWindow* self);
VALA_EXTERN TunerDisplay* tuner_display_new (TunerDirectoryController* directory);
VALA_EXTERN TunerDisplay* tuner_display_construct (GType object_type,
                                       TunerDirectoryController* directory);
VALA_EXTERN GType tuner_model_favicon_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerModelFavicon, g_object_unref)
VALA_EXTERN GType tuner_model_station_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerModelStation, g_object_unref)
VALA_EXTERN void tuner_window_handle_play_station (TunerWindow* self,
                                       TunerModelStation* station);
static void _tuner_window_handle_play_station_tuner_display_station_clicked_sig (TunerDisplay* _sender,
                                                                          TunerModelStation* station,
                                                                          gpointer self);
VALA_EXTERN gboolean tuner_settings_get_auto_play (TunerSettings* self);
VALA_EXTERN void tuner_directory_controller_load (TunerDirectoryController* self);
VALA_EXTERN GType tuner_station_set_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerStationSet, g_object_unref)
VALA_EXTERN TunerStationSet* tuner_directory_controller_load_station_uuid (TunerDirectoryController* self,
                                                               const gchar* uuid);
VALA_EXTERN const gchar* tuner_settings_get_last_played_station (TunerSettings* self);
VALA_EXTERN GQuark source_error_quark (void);
GType source_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GeeSet* tuner_station_set_next_page (TunerStationSet* self,
                                     GError** error);
VALA_EXTERN GType tuner_about_dialog_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TunerAboutDialog, g_object_unref)
VALA_EXTERN TunerAboutDialog* tuner_about_dialog_new (GtkWindow* window);
VALA_EXTERN TunerAboutDialog* tuner_about_dialog_construct (GType object_type,
                                                GtkWindow* window);
VALA_EXTERN TunerPlayerController* tuner_window_get_player_ctrl (TunerWindow* self);
VALA_EXTERN void tuner_player_controller_play_pause (TunerPlayerController* self);
VALA_EXTERN TunerSettings* tuner_window_get_settings (TunerWindow* self);
VALA_EXTERN gboolean tuner_settings_get_do_not_vote (TunerSettings* self);
VALA_EXTERN void tuner_settings_set_do_not_vote (TunerSettings* self,
                                     gboolean value);
static GVariant* _variant_new6 (gboolean value);
VALA_EXTERN void tuner_settings_set_auto_play (TunerSettings* self,
                                   gboolean value);
static GVariant* _variant_new7 (gboolean value);
VALA_EXTERN void tuner_settings_set_start_on_starred (TunerSettings* self,
                                          gboolean value);
static GVariant* _variant_new8 (gboolean value);
VALA_EXTERN gboolean tuner_settings_get_stream_info (TunerSettings* self);
VALA_EXTERN void tuner_settings_set_stream_info (TunerSettings* self,
                                     gboolean value);
static GVariant* _variant_new9 (gboolean value);
VALA_EXTERN void tuner_header_bar_stream_info (TunerHeaderBar* self,
                                   gboolean show);
VALA_EXTERN gboolean tuner_settings_get_stream_info_fast (TunerSettings* self);
VALA_EXTERN void tuner_settings_set_stream_info_fast (TunerSettings* self,
                                          gboolean value);
static GVariant* _variant_new10 (gboolean value);
VALA_EXTERN void tuner_header_bar_stream_info_fast (TunerHeaderBar* self,
                                        gboolean fast);
VALA_EXTERN TunerApplication* tuner_app (void);
VALA_EXTERN gboolean tuner_application_get_is_offline (TunerApplication* self);
VALA_EXTERN gboolean tuner_header_bar_update_playing_station (TunerHeaderBar* self,
                                                  TunerModelStation* station);
VALA_EXTERN void tuner_player_controller_set_station (TunerPlayerController* self,
                                          TunerModelStation* value);
VALA_EXTERN const gchar* tuner_model_station_get_stationuuid (TunerModelStation* self);
VALA_EXTERN void tuner_settings_set_last_played_station (TunerSettings* self,
                                             const gchar* value);
VALA_EXTERN void tuner_directory_controller_count_station_click (TunerDirectoryController* self,
                                                     TunerModelStation* station);
VALA_EXTERN const gchar* tuner_model_station_get_name (TunerModelStation* self);
VALA_EXTERN gboolean tuner_window_before_destroy (TunerWindow* self);
VALA_EXTERN void tuner_settings_save (TunerSettings* self);
VALA_EXTERN GType tuner_player_controller_is_get_type (void) G_GNUC_CONST ;
VALA_EXTERN TunerPlayerControllerIs tuner_player_controller_get_player_state (TunerPlayerController* self);
VALA_EXTERN gboolean tuner_window_get_active (TunerWindow* self);
static void tuner_window_set_active (TunerWindow* self,
                              gboolean value);
VALA_EXTERN gboolean tuner_application_get_is_online (TunerApplication* self);
VALA_EXTERN void tuner_display_update_state (TunerDisplay* self,
                                 gboolean activate,
                                 gboolean start_on_starred);
static void tuner_window_set_settings (TunerWindow* self,
                                TunerSettings* value);
static void tuner_window_set_player_ctrl (TunerWindow* self,
                                   TunerPlayerController* value);
static void tuner_window_set_directory (TunerWindow* self,
                                 TunerDirectoryController* value);
VALA_EXTERN gint tuner_window_get_width (TunerWindow* self);
static void tuner_window_set_width (TunerWindow* self,
                             gint value);
VALA_EXTERN gint tuner_window_get_height (TunerWindow* self);
static void tuner_window_set_height (TunerWindow* self,
                              gint value);
static void g_cclosure_user_marshal_VOID__BOOLEAN_STRING (GClosure * closure,
                                                   GValue * return_value,
                                                   guint n_param_values,
                                                   const GValue * param_values,
                                                   gpointer invocation_hint,
                                                   gpointer marshal_data);
static GObject * tuner_window_constructor (GType type,
                                    guint n_construct_properties,
                                    GObjectConstructParam * construct_properties);
static GVariant* _variant_new11 (gboolean value);
static GVariant* _variant_new12 (gboolean value);
static GVariant* _variant_new13 (gboolean value);
static GVariant* _variant_new14 (gboolean value);
static GVariant* _variant_new15 (gboolean value);
static gboolean _tuner_window___lambda88_ (TunerWindow* self,
                                    GdkEventAny* e);
static gboolean __tuner_window___lambda88__gtk_widget_delete_event (GtkWidget* _sender,
                                                             GdkEventAny* event,
                                                             gpointer self);
static void _tuner_window___lambda89_ (TunerWindow* self);
static void __tuner_window___lambda89__g_object_notify (GObject* _sender,
                                                 GParamSpec* pspec,
                                                 gpointer self);
static void tuner_window_finalize (GObject * obj);
static GType tuner_window_get_type_once (void);
static void _vala_tuner_window_get_property (GObject * object,
                                      guint property_id,
                                      GValue * value,
                                      GParamSpec * pspec);
static void _vala_tuner_window_set_property (GObject * object,
                                      guint property_id,
                                      const GValue * value,
                                      GParamSpec * pspec);
static void _vala_array_destroy (gpointer array,
                          gssize array_length,
                          GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array,
                       gssize array_length,
                       GDestroyNotify destroy_func);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static const GActionEntry TUNER_WINDOW_ACTION_ENTRIES[8] = {{TUNER_WINDOW_ACTION_PAUSE, _tuner_window_on_toggle_playback_gsimple_action_activate_callback}, {TUNER_WINDOW_ACTION_QUIT, _tuner_window_on_action_quit_gsimple_action_activate_callback}, {TUNER_WINDOW_ACTION_ABOUT, _tuner_window_on_action_about_gsimple_action_activate_callback}, {TUNER_WINDOW_ACTION_DISABLE_TRACKING, _tuner_window_on_action_disable_tracking_gsimple_action_activate_callback, NULL, "false"}, {TUNER_WINDOW_ACTION_ENABLE_AUTOPLAY, _tuner_window_on_action_enable_autoplay_gsimple_action_activate_callback, NULL, "false"}, {TUNER_WINDOW_ACTION_START_ON_STARRED, _tuner_window_on_action_start_on_starred_gsimple_action_activate_callback, NULL, "false"}, {TUNER_WINDOW_ACTION_STREAM_INFO, _tuner_window_on_action_stream_info_gsimple_action_activate_callback, NULL, "true"}, {TUNER_WINDOW_ACTION_STREAM_INFO_FAST, _tuner_window_on_action_stream_info_fast_gsimple_action_activate_callback, NULL, "false"}};

static inline gpointer
tuner_window_get_instance_private (TunerWindow* self)
{
	return G_STRUCT_MEMBER_P (self, TunerWindow_private_offset);
}

static void
_tuner_window_on_toggle_playback_gsimple_action_activate_callback (GSimpleAction* action,
                                                                   GVariant* parameter,
                                                                   gpointer self)
{
	tuner_window_on_toggle_playback ((TunerWindow*) self);
}

static void
_tuner_window_on_action_quit_gsimple_action_activate_callback (GSimpleAction* action,
                                                               GVariant* parameter,
                                                               gpointer self)
{
	tuner_window_on_action_quit ((TunerWindow*) self);
}

static void
_tuner_window_on_action_about_gsimple_action_activate_callback (GSimpleAction* action,
                                                                GVariant* parameter,
                                                                gpointer self)
{
	tuner_window_on_action_about ((TunerWindow*) self);
}

static void
_tuner_window_on_action_disable_tracking_gsimple_action_activate_callback (GSimpleAction* action,
                                                                           GVariant* parameter,
                                                                           gpointer self)
{
	tuner_window_on_action_disable_tracking ((TunerWindow*) self, action, parameter);
}

static void
_tuner_window_on_action_enable_autoplay_gsimple_action_activate_callback (GSimpleAction* action,
                                                                          GVariant* parameter,
                                                                          gpointer self)
{
	tuner_window_on_action_enable_autoplay ((TunerWindow*) self, action, parameter);
}

static void
_tuner_window_on_action_start_on_starred_gsimple_action_activate_callback (GSimpleAction* action,
                                                                           GVariant* parameter,
                                                                           gpointer self)
{
	tuner_window_on_action_start_on_starred ((TunerWindow*) self, action, parameter);
}

static void
_tuner_window_on_action_stream_info_gsimple_action_activate_callback (GSimpleAction* action,
                                                                      GVariant* parameter,
                                                                      gpointer self)
{
	tuner_window_on_action_stream_info ((TunerWindow*) self, action, parameter);
}

static void
_tuner_window_on_action_stream_info_fast_gsimple_action_activate_callback (GSimpleAction* action,
                                                                           GVariant* parameter,
                                                                           gpointer self)
{
	tuner_window_on_action_stream_info_fast ((TunerWindow*) self, action, parameter);
}

/**
     * @brief Constructs a new Window instance.
     *
     * @param app The Application instance.
     * @param player The PlayerController instance.
     */
TunerWindow*
tuner_window_construct (GType object_type,
                        TunerApplication* app,
                        TunerPlayerController* player,
                        TunerSettings* settings,
                        TunerDirectoryController* directory)
{
	TunerWindow * self = NULL;
	gboolean _tmp0_;
	gboolean _tmp1_;
	GtkApplication* _tmp2_;
	GtkApplication* _tmp3_;
	gchar* _tmp4_;
	gchar** _tmp5_;
	gchar** _tmp6_;
	gint _tmp6__length1;
	GtkApplication* _tmp7_;
	GtkApplication* _tmp8_;
	gchar* _tmp9_;
	gchar** _tmp10_;
	gchar** _tmp11_;
	gint _tmp11__length1;
	GtkApplication* _tmp12_;
	GtkApplication* _tmp13_;
	gchar* _tmp14_;
	gchar** _tmp15_;
	gchar** _tmp16_;
	gint _tmp16__length1;
	g_return_val_if_fail (app != NULL, NULL);
	g_return_val_if_fail (player != NULL, NULL);
	g_return_val_if_fail (settings != NULL, NULL);
	g_return_val_if_fail (directory != NULL, NULL);
	self = (TunerWindow*) g_object_new (object_type, "application", app, "player-ctrl", player, "settings", settings, "directory", directory, NULL);
	tuner_window_add_widgets (self);
	tuner_window_check_online_status (self);
	_tmp0_ = tuner_settings_get_start_on_starred (settings);
	_tmp1_ = _tmp0_;
	if (_tmp1_) {
		tuner_window_choose_starred_stations (self);
	}
	gtk_widget_show_all ((GtkWidget*) self);
	_tmp2_ = gtk_window_get_application ((GtkWindow*) self);
	_tmp3_ = _tmp2_;
	_tmp4_ = g_strdup ("<Control>5");
	_tmp5_ = g_new0 (gchar*, 1 + 1);
	_tmp5_[0] = _tmp4_;
	_tmp6_ = _tmp5_;
	_tmp6__length1 = 1;
	gtk_application_set_accels_for_action (_tmp3_, TUNER_WINDOW_ACTION_PREFIX TUNER_WINDOW_ACTION_PAUSE, _tmp6_);
	_tmp6_ = (_vala_array_free (_tmp6_, _tmp6__length1, (GDestroyNotify) g_free), NULL);
	_tmp7_ = gtk_window_get_application ((GtkWindow*) self);
	_tmp8_ = _tmp7_;
	_tmp9_ = g_strdup ("<Control>q");
	_tmp10_ = g_new0 (gchar*, 1 + 1);
	_tmp10_[0] = _tmp9_;
	_tmp11_ = _tmp10_;
	_tmp11__length1 = 1;
	gtk_application_set_accels_for_action (_tmp8_, TUNER_WINDOW_ACTION_PREFIX TUNER_WINDOW_ACTION_QUIT, _tmp11_);
	_tmp11_ = (_vala_array_free (_tmp11_, _tmp11__length1, (GDestroyNotify) g_free), NULL);
	_tmp12_ = gtk_window_get_application ((GtkWindow*) self);
	_tmp13_ = _tmp12_;
	_tmp14_ = g_strdup ("<Control>w");
	_tmp15_ = g_new0 (gchar*, 1 + 1);
	_tmp15_[0] = _tmp14_;
	_tmp16_ = _tmp15_;
	_tmp16__length1 = 1;
	gtk_application_set_accels_for_action (_tmp13_, TUNER_WINDOW_ACTION_PREFIX TUNER_WINDOW_ACTION_QUIT, _tmp16_);
	_tmp16_ = (_vala_array_free (_tmp16_, _tmp16__length1, (GDestroyNotify) g_free), NULL);
	return self;
}

TunerWindow*
tuner_window_new (TunerApplication* app,
                  TunerPlayerController* player,
                  TunerSettings* settings,
                  TunerDirectoryController* directory)
{
	return tuner_window_construct (TUNER_TYPE_WINDOW, app, player, settings, directory);
}

/**
     * Selects and displays the user's starred (favorite) radio stations.
     * 
     * This method handles the process of showing the user's favorite stations
     * in the station list view. It filters and displays only the stations that
     * have been marked as favorites by the user.
     */
void
tuner_window_choose_starred_stations (TunerWindow* self)
{
	g_return_if_fail (self != NULL);
	self->priv->_start_on_starred = TRUE;
	if (self->priv->_active) {
		TunerDisplay* _tmp0_;
		_tmp0_ = self->priv->_display;
		tuner_display_choose_starred_stations (_tmp0_);
	}
}

/**
        Add widgets after Window creation
    */
static void
__lambda86_ (TunerWindow* self)
{
	TunerDisplay* _tmp0_;
	_tmp0_ = self->priv->_display;
	g_signal_emit_by_name (_tmp0_, "search-focused-sig");
}

static void
___lambda86__tuner_header_bar_search_has_focus_sig (TunerHeaderBar* _sender,
                                                    gpointer self)
{
	__lambda86_ ((TunerWindow*) self);
}

static void
__lambda87_ (TunerWindow* self,
             const gchar* text)
{
	TunerDisplay* _tmp0_;
	g_return_if_fail (text != NULL);
	_tmp0_ = self->priv->_display;
	g_signal_emit_by_name (_tmp0_, "searched-for-sig", text);
}

static void
___lambda87__tuner_header_bar_searching_for_sig (TunerHeaderBar* _sender,
                                                 const gchar* text,
                                                 gpointer self)
{
	__lambda87_ ((TunerWindow*) self, text);
}

static void
_tuner_window_handle_play_station_tuner_display_station_clicked_sig (TunerDisplay* _sender,
                                                                     TunerModelStation* station,
                                                                     gpointer self)
{
	tuner_window_handle_play_station ((TunerWindow*) self, station);
}

static void
tuner_window_add_widgets (TunerWindow* self)
{
	TunerHeaderBar* _tmp0_;
	TunerHeaderBar* _tmp1_;
	TunerHeaderBar* _tmp2_;
	TunerHeaderBar* _tmp3_;
	TunerDirectoryController* _tmp4_;
	TunerDisplay* _tmp5_;
	TunerDisplay* _tmp6_;
	TunerDisplay* _tmp7_;
	TunerSettings* _tmp8_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	_tmp0_ = tuner_header_bar_new (self);
	g_object_ref_sink (_tmp0_);
	_g_object_unref0 (self->priv->_headerbar);
	self->priv->_headerbar = _tmp0_;
	_tmp1_ = self->priv->_headerbar;
	g_signal_connect_object (_tmp1_, "search-has-focus-sig", (GCallback) ___lambda86__tuner_header_bar_search_has_focus_sig, self, 0);
	_tmp2_ = self->priv->_headerbar;
	g_signal_connect_object (_tmp2_, "searching-for-sig", (GCallback) ___lambda87__tuner_header_bar_searching_for_sig, self, 0);
	_tmp3_ = self->priv->_headerbar;
	gtk_window_set_titlebar ((GtkWindow*) self, (GtkWidget*) _tmp3_);
	_tmp4_ = self->priv->_directory;
	_tmp5_ = tuner_display_new (_tmp4_);
	g_object_ref_sink (_tmp5_);
	_g_object_unref0 (self->priv->_display);
	self->priv->_display = _tmp5_;
	_tmp6_ = self->priv->_display;
	g_signal_connect_object (_tmp6_, "station-clicked-sig", (GCallback) _tuner_window_handle_play_station_tuner_display_station_clicked_sig, self, 0);
	_tmp7_ = self->priv->_display;
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) _tmp7_);
	_tmp8_ = self->priv->_settings;
	_tmp9_ = tuner_settings_get_auto_play (_tmp8_);
	_tmp10_ = _tmp9_;
	if (_tmp10_) {
		TunerDirectoryController* _tmp11_;
		TunerStationSet* source = NULL;
		TunerDirectoryController* _tmp12_;
		TunerSettings* _tmp13_;
		const gchar* _tmp14_;
		const gchar* _tmp15_;
		TunerStationSet* _tmp16_;
		_tmp11_ = self->priv->_directory;
		tuner_directory_controller_load (_tmp11_);
		_tmp12_ = self->priv->_directory;
		_tmp13_ = self->priv->_settings;
		_tmp14_ = tuner_settings_get_last_played_station (_tmp13_);
		_tmp15_ = _tmp14_;
		_tmp16_ = tuner_directory_controller_load_station_uuid (_tmp12_, _tmp15_);
		source = _tmp16_;
		{
			GeeSet* _tmp17_ = NULL;
			TunerStationSet* _tmp18_;
			GeeSet* _tmp19_;
			_tmp18_ = source;
			_tmp19_ = tuner_station_set_next_page (_tmp18_, &_inner_error0_);
			_tmp17_ = _tmp19_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == SOURCE_ERROR) {
					goto __catch0_source_error;
				}
				_g_object_unref0 (source);
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
			{
				GeeIterator* _station_it = NULL;
				GeeIterator* _tmp20_;
				_tmp20_ = gee_iterable_iterator ((GeeIterable*) _tmp17_);
				_station_it = _tmp20_;
				while (TRUE) {
					GeeIterator* _tmp21_;
					TunerModelStation* station = NULL;
					GeeIterator* _tmp22_;
					gpointer _tmp23_;
					TunerModelStation* _tmp24_;
					_tmp21_ = _station_it;
					if (!gee_iterator_next (_tmp21_)) {
						break;
					}
					_tmp22_ = _station_it;
					_tmp23_ = gee_iterator_get (_tmp22_);
					station = (TunerModelStation*) _tmp23_;
					_tmp24_ = station;
					tuner_window_handle_play_station (self, _tmp24_);
					_g_object_unref0 (station);
					break;
				}
				_g_object_unref0 (_station_it);
			}
			_g_object_unref0 (_tmp17_);
		}
		goto __finally0;
		__catch0_source_error:
		{
			g_clear_error (&_inner_error0_);
			g_warning ("Window.vala:242: Error while trying to autoplay, aborting...");
		}
		__finally0:
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			_g_object_unref0 (source);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
		_g_object_unref0 (source);
	}
}

/**
     * @brief Handles the quit action.
     */
static void
tuner_window_on_action_quit (TunerWindow* self)
{
	g_return_if_fail (self != NULL);
	gtk_window_close ((GtkWindow*) self);
}

/**
     * @brief Handles the about action.
     */
static void
tuner_window_on_action_about (TunerWindow* self)
{
	TunerAboutDialog* dialog = NULL;
	TunerAboutDialog* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = tuner_about_dialog_new ((GtkWindow*) self);
	g_object_ref_sink (_tmp0_);
	dialog = _tmp0_;
	gtk_window_present ((GtkWindow*) dialog);
	_g_object_unref0 (dialog);
}

/**
     * @brief Toggles playback state.
     */
void
tuner_window_on_toggle_playback (TunerWindow* self)
{
	TunerPlayerController* _tmp0_;
	g_return_if_fail (self != NULL);
	g_info ("Window.vala:286: %s", _ ("Stop Playback requested"));
	_tmp0_ = self->priv->_player_ctrl;
	tuner_player_controller_play_pause (_tmp0_);
}

/**
     * @brief Handles the disable tracking action.
     * @param action The SimpleAction that triggered this method.
     * @param parameter The parameter passed with the action (unused).
     */
static GVariant*
_variant_new6 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

static gchar*
bool_to_string (gboolean self)
{
	gchar* result;
	if (self) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup ("true");
		result = _tmp0_;
		return result;
	} else {
		gchar* _tmp1_;
		_tmp1_ = g_strdup ("false");
		result = _tmp1_;
		return result;
	}
}

void
tuner_window_on_action_disable_tracking (TunerWindow* self,
                                         GSimpleAction* action,
                                         GVariant* parameter)
{
	TunerSettings* _tmp0_;
	TunerSettings* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	TunerSettings* _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GVariant* _tmp7_;
	TunerSettings* _tmp8_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	gchar* _tmp11_;
	gchar* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0_ = self->priv->_settings;
	_tmp1_ = self->priv->_settings;
	_tmp2_ = tuner_settings_get_do_not_vote (_tmp1_);
	_tmp3_ = _tmp2_;
	tuner_settings_set_do_not_vote (_tmp0_, !_tmp3_);
	_tmp4_ = self->priv->_settings;
	_tmp5_ = tuner_settings_get_do_not_vote (_tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = _variant_new6 (_tmp6_);
	g_simple_action_set_state (action, _tmp7_);
	_g_variant_unref0 (_tmp7_);
	_tmp8_ = self->priv->_settings;
	_tmp9_ = tuner_settings_get_do_not_vote (_tmp8_);
	_tmp10_ = _tmp9_;
	_tmp11_ = bool_to_string (_tmp10_);
	_tmp12_ = _tmp11_;
	_tmp13_ = g_strconcat ("on_action_disable_tracking: ", _tmp12_, NULL);
	_tmp14_ = _tmp13_;
	g_debug ("Window.vala:300: %s", _tmp14_);
	_g_free0 (_tmp14_);
	_g_free0 (_tmp12_);
}

/**
     * @brief Handles the enable autoplay action.
     * @param action The SimpleAction that triggered this method.
     * @param parameter The parameter passed with the action (unused).
     */
static GVariant*
_variant_new7 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

void
tuner_window_on_action_enable_autoplay (TunerWindow* self,
                                        GSimpleAction* action,
                                        GVariant* parameter)
{
	TunerSettings* _tmp0_;
	TunerSettings* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	TunerSettings* _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GVariant* _tmp7_;
	TunerSettings* _tmp8_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	gchar* _tmp11_;
	gchar* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0_ = self->priv->_settings;
	_tmp1_ = self->priv->_settings;
	_tmp2_ = tuner_settings_get_auto_play (_tmp1_);
	_tmp3_ = _tmp2_;
	tuner_settings_set_auto_play (_tmp0_, !_tmp3_);
	_tmp4_ = self->priv->_settings;
	_tmp5_ = tuner_settings_get_auto_play (_tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = _variant_new7 (_tmp6_);
	g_simple_action_set_state (action, _tmp7_);
	_g_variant_unref0 (_tmp7_);
	_tmp8_ = self->priv->_settings;
	_tmp9_ = tuner_settings_get_auto_play (_tmp8_);
	_tmp10_ = _tmp9_;
	_tmp11_ = bool_to_string (_tmp10_);
	_tmp12_ = _tmp11_;
	_tmp13_ = g_strconcat ("on_action_enable_autoplay: ", _tmp12_, NULL);
	_tmp14_ = _tmp13_;
	g_debug ("Window.vala:313: %s", _tmp14_);
	_g_free0 (_tmp14_);
	_g_free0 (_tmp12_);
}

/**
     * @brief Handles the enable autoplay action.
     * @param action The SimpleAction that triggered this method.
     * @param parameter The parameter passed with the action (unused).
     */
static GVariant*
_variant_new8 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

void
tuner_window_on_action_start_on_starred (TunerWindow* self,
                                         GSimpleAction* action,
                                         GVariant* parameter)
{
	TunerSettings* _tmp0_;
	TunerSettings* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	TunerSettings* _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GVariant* _tmp7_;
	TunerSettings* _tmp8_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	gchar* _tmp11_;
	gchar* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0_ = self->priv->_settings;
	_tmp1_ = self->priv->_settings;
	_tmp2_ = tuner_settings_get_start_on_starred (_tmp1_);
	_tmp3_ = _tmp2_;
	tuner_settings_set_start_on_starred (_tmp0_, !_tmp3_);
	_tmp4_ = self->priv->_settings;
	_tmp5_ = tuner_settings_get_start_on_starred (_tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = _variant_new8 (_tmp6_);
	g_simple_action_set_state (action, _tmp7_);
	_g_variant_unref0 (_tmp7_);
	_tmp8_ = self->priv->_settings;
	_tmp9_ = tuner_settings_get_auto_play (_tmp8_);
	_tmp10_ = _tmp9_;
	_tmp11_ = bool_to_string (_tmp10_);
	_tmp12_ = _tmp11_;
	_tmp13_ = g_strconcat ("on_action_enable_autoplay: ", _tmp12_, NULL);
	_tmp14_ = _tmp13_;
	g_debug ("Window.vala:326: %s", _tmp14_);
	_g_free0 (_tmp14_);
	_g_free0 (_tmp12_);
}

static GVariant*
_variant_new9 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

void
tuner_window_on_action_stream_info (TunerWindow* self,
                                    GSimpleAction* action,
                                    GVariant* parameter)
{
	TunerSettings* _tmp0_;
	TunerSettings* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	TunerSettings* _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GVariant* _tmp7_;
	TunerHeaderBar* _tmp8_;
	GVariant* _tmp9_;
	GVariant* _tmp10_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0_ = self->priv->_settings;
	_tmp1_ = self->priv->_settings;
	_tmp2_ = tuner_settings_get_stream_info (_tmp1_);
	_tmp3_ = _tmp2_;
	tuner_settings_set_stream_info (_tmp0_, !_tmp3_);
	_tmp4_ = self->priv->_settings;
	_tmp5_ = tuner_settings_get_stream_info (_tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = _variant_new9 (_tmp6_);
	g_simple_action_set_state (action, _tmp7_);
	_g_variant_unref0 (_tmp7_);
	_tmp8_ = self->priv->_headerbar;
	_tmp9_ = g_action_get_state ((GAction*) action);
	_tmp10_ = _tmp9_;
	tuner_header_bar_stream_info (_tmp8_, g_variant_get_boolean (_tmp10_));
	_g_variant_unref0 (_tmp10_);
}

static GVariant*
_variant_new10 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

void
tuner_window_on_action_stream_info_fast (TunerWindow* self,
                                         GSimpleAction* action,
                                         GVariant* parameter)
{
	TunerSettings* _tmp0_;
	TunerSettings* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	TunerSettings* _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GVariant* _tmp7_;
	TunerHeaderBar* _tmp8_;
	GVariant* _tmp9_;
	GVariant* _tmp10_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0_ = self->priv->_settings;
	_tmp1_ = self->priv->_settings;
	_tmp2_ = tuner_settings_get_stream_info_fast (_tmp1_);
	_tmp3_ = _tmp2_;
	tuner_settings_set_stream_info_fast (_tmp0_, !_tmp3_);
	_tmp4_ = self->priv->_settings;
	_tmp5_ = tuner_settings_get_stream_info_fast (_tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = _variant_new10 (_tmp6_);
	g_simple_action_set_state (action, _tmp7_);
	_g_variant_unref0 (_tmp7_);
	_tmp8_ = self->priv->_headerbar;
	_tmp9_ = g_action_get_state ((GAction*) action);
	_tmp10_ = _tmp9_;
	tuner_header_bar_stream_info_fast (_tmp8_, g_variant_get_boolean (_tmp10_));
	_g_variant_unref0 (_tmp10_);
}

/**
* @brief Handles a station selection and plays the station
* @param station The selected station.
*/
void
tuner_window_handle_play_station (TunerWindow* self,
                                  TunerModelStation* station)
{
	gboolean _tmp0_ = FALSE;
	TunerApplication* _tmp1_;
	TunerApplication* _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
	TunerPlayerController* _tmp7_;
	TunerSettings* _tmp8_;
	const gchar* _tmp9_;
	const gchar* _tmp10_;
	TunerDirectoryController* _tmp11_;
	const gchar* _tmp12_;
	const gchar* _tmp13_;
	gchar* _tmp14_;
	gchar* _tmp15_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (station != NULL);
	_tmp1_ = tuner_app ();
	_tmp2_ = _tmp1_;
	_tmp3_ = tuner_application_get_is_offline (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = _tmp4_;
	_g_object_unref0 (_tmp2_);
	if (_tmp5_) {
		_tmp0_ = TRUE;
	} else {
		TunerHeaderBar* _tmp6_;
		_tmp6_ = self->priv->_headerbar;
		_tmp0_ = !tuner_header_bar_update_playing_station (_tmp6_, station);
	}
	if (_tmp0_) {
		return;
	}
	_tmp7_ = self->priv->_player_ctrl;
	tuner_player_controller_set_station (_tmp7_, station);
	_tmp8_ = self->priv->_settings;
	_tmp9_ = tuner_model_station_get_stationuuid (station);
	_tmp10_ = _tmp9_;
	tuner_settings_set_last_played_station (_tmp8_, _tmp10_);
	_tmp11_ = self->priv->_directory;
	tuner_directory_controller_count_station_click (_tmp11_, station);
	_tmp12_ = tuner_model_station_get_name (station);
	_tmp13_ = _tmp12_;
	_tmp14_ = g_strconcat (TUNER_WINDOW_WINDOW_NAME ": ", _tmp13_, NULL);
	_tmp15_ = _tmp14_;
	gtk_window_set_title ((GtkWindow*) self, _tmp15_);
	_g_free0 (_tmp15_);
}

/**
* @brief Performs cleanup actions before the window is destroyed.
* @return true if the window should be hidden instead of destroyed, false otherwise.
*/
gboolean
tuner_window_before_destroy (TunerWindow* self)
{
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	TunerSettings* _tmp2_;
	TunerPlayerController* _tmp3_;
	TunerPlayerControllerIs _tmp4_;
	TunerPlayerControllerIs _tmp5_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	gtk_window_get_size ((GtkWindow*) self, &_tmp0_, &_tmp1_);
	self->priv->_width = _tmp0_;
	self->priv->_height = _tmp1_;
	_tmp2_ = self->priv->_settings;
	tuner_settings_save (_tmp2_);
	_tmp3_ = self->priv->_player_ctrl;
	_tmp4_ = tuner_player_controller_get_player_state (_tmp3_);
	_tmp5_ = _tmp4_;
	if (_tmp5_ == TUNER_PLAYER_CONTROLLER_IS_PLAYING) {
		GNotification* notification = NULL;
		GNotification* _tmp6_;
		GNotification* _tmp7_;
		GNotification* _tmp8_;
		TunerApplication* _tmp9_;
		TunerApplication* _tmp10_;
		GNotification* _tmp11_;
		gtk_widget_hide_on_delete ((GtkWidget*) self);
		_tmp6_ = g_notification_new (TUNER_WINDOW_NOTIFICATION_PLAYING_BACKGROUND);
		notification = _tmp6_;
		_tmp7_ = notification;
		g_notification_set_body (_tmp7_, TUNER_WINDOW_NOTIFICATION_CLICK_RESUME);
		_tmp8_ = notification;
		g_notification_set_default_action (_tmp8_, TUNER_WINDOW_NOTIFICATION_APP_RESUME_WINDOW);
		_tmp9_ = tuner_app ();
		_tmp10_ = _tmp9_;
		_tmp11_ = notification;
		g_application_send_notification ((GApplication*) _tmp10_, TUNER_WINDOW_NOTIFICATION_APP_PLAYING_CONTINUE, _tmp11_);
		_g_object_unref0 (_tmp10_);
		result = TRUE;
		_g_object_unref0 (notification);
		return result;
	}
	result = FALSE;
	return result;
}

/**
* @brief Checks changes in online state and updates the app accordingly
*
*/
static void
tuner_window_check_online_status (TunerWindow* self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_;
	gboolean _tmp6_ = FALSE;
	gboolean _tmp7_;
	TunerDisplay* _tmp12_;
	gboolean _tmp13_;
	g_return_if_fail (self != NULL);
	_tmp1_ = self->priv->_active;
	if (_tmp1_) {
		TunerApplication* _tmp2_;
		TunerApplication* _tmp3_;
		gboolean _tmp4_;
		gboolean _tmp5_;
		_tmp2_ = tuner_app ();
		_tmp3_ = _tmp2_;
		_tmp4_ = tuner_application_get_is_offline (_tmp3_);
		_tmp5_ = _tmp4_;
		_tmp0_ = _tmp5_;
		_g_object_unref0 (_tmp3_);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gtk_window_set_accept_focus ((GtkWindow*) self, FALSE);
		tuner_window_set_active (self, FALSE);
	}
	_tmp7_ = self->priv->_active;
	if (!_tmp7_) {
		TunerApplication* _tmp8_;
		TunerApplication* _tmp9_;
		gboolean _tmp10_;
		gboolean _tmp11_;
		_tmp8_ = tuner_app ();
		_tmp9_ = _tmp8_;
		_tmp10_ = tuner_application_get_is_online (_tmp9_);
		_tmp11_ = _tmp10_;
		_tmp6_ = _tmp11_;
		_g_object_unref0 (_tmp9_);
	} else {
		_tmp6_ = FALSE;
	}
	if (_tmp6_) {
		gtk_window_set_accept_focus ((GtkWindow*) self, TRUE);
		tuner_window_set_active (self, TRUE);
	}
	_tmp12_ = self->priv->_display;
	_tmp13_ = self->priv->_active;
	tuner_display_update_state (_tmp12_, _tmp13_, self->priv->_start_on_starred);
}

TunerSettings*
tuner_window_get_settings (TunerWindow* self)
{
	TunerSettings* result;
	TunerSettings* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_settings;
	result = _tmp0_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
tuner_window_set_settings (TunerWindow* self,
                           TunerSettings* value)
{
	TunerSettings* old_value;
	g_return_if_fail (self != NULL);
	old_value = tuner_window_get_settings (self);
	if (old_value != value) {
		TunerSettings* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_settings);
		self->priv->_settings = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, tuner_window_properties[TUNER_WINDOW_SETTINGS_PROPERTY]);
	}
}

TunerPlayerController*
tuner_window_get_player_ctrl (TunerWindow* self)
{
	TunerPlayerController* result;
	TunerPlayerController* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_player_ctrl;
	result = _tmp0_;
	return result;
}

static void
tuner_window_set_player_ctrl (TunerWindow* self,
                              TunerPlayerController* value)
{
	TunerPlayerController* old_value;
	g_return_if_fail (self != NULL);
	old_value = tuner_window_get_player_ctrl (self);
	if (old_value != value) {
		TunerPlayerController* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_player_ctrl);
		self->priv->_player_ctrl = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, tuner_window_properties[TUNER_WINDOW_PLAYER_CTRL_PROPERTY]);
	}
}

TunerDirectoryController*
tuner_window_get_directory (TunerWindow* self)
{
	TunerDirectoryController* result;
	TunerDirectoryController* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_directory;
	result = _tmp0_;
	return result;
}

static void
tuner_window_set_directory (TunerWindow* self,
                            TunerDirectoryController* value)
{
	TunerDirectoryController* old_value;
	g_return_if_fail (self != NULL);
	old_value = tuner_window_get_directory (self);
	if (old_value != value) {
		TunerDirectoryController* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_directory);
		self->priv->_directory = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, tuner_window_properties[TUNER_WINDOW_DIRECTORY_PROPERTY]);
	}
}

gboolean
tuner_window_get_active (TunerWindow* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_active;
	return result;
}

static void
tuner_window_set_active (TunerWindow* self,
                         gboolean value)
{
	gboolean old_value;
	g_return_if_fail (self != NULL);
	old_value = tuner_window_get_active (self);
	if (old_value != value) {
		self->priv->_active = value;
		g_object_notify_by_pspec ((GObject *) self, tuner_window_properties[TUNER_WINDOW_ACTIVE_PROPERTY]);
	}
}

gint
tuner_window_get_width (TunerWindow* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_width;
	return result;
}

static void
tuner_window_set_width (TunerWindow* self,
                        gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = tuner_window_get_width (self);
	if (old_value != value) {
		self->priv->_width = value;
		g_object_notify_by_pspec ((GObject *) self, tuner_window_properties[TUNER_WINDOW_WIDTH_PROPERTY]);
	}
}

gint
tuner_window_get_height (TunerWindow* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_height;
	return result;
}

static void
tuner_window_set_height (TunerWindow* self,
                         gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = tuner_window_get_height (self);
	if (old_value != value) {
		self->priv->_height = value;
		g_object_notify_by_pspec ((GObject *) self, tuner_window_properties[TUNER_WINDOW_HEIGHT_PROPERTY]);
	}
}

static void
g_cclosure_user_marshal_VOID__BOOLEAN_STRING (GClosure * closure,
                                              GValue * return_value,
                                              guint n_param_values,
                                              const GValue * param_values,
                                              gpointer invocation_hint,
                                              gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__BOOLEAN_STRING) (gpointer data1, gboolean arg_1, const char* arg_2, gpointer data2);
	register GMarshalFunc_VOID__BOOLEAN_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__BOOLEAN_STRING) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_boolean (param_values + 1), g_value_get_string (param_values + 2), data2);
}

static GVariant*
_variant_new11 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

static GVariant*
_variant_new12 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

static GVariant*
_variant_new13 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

static GVariant*
_variant_new14 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

static GVariant*
_variant_new15 (gboolean value)
{
	return g_variant_ref_sink (g_variant_new_boolean (value));
}

static gboolean
_tuner_window___lambda88_ (TunerWindow* self,
                           GdkEventAny* e)
{
	gboolean result;
	g_return_val_if_fail (e != NULL, FALSE);
	result = tuner_window_before_destroy (self);
	return result;
}

static gboolean
__tuner_window___lambda88__gtk_widget_delete_event (GtkWidget* _sender,
                                                    GdkEventAny* event,
                                                    gpointer self)
{
	gboolean result;
	result = _tuner_window___lambda88_ ((TunerWindow*) self, event);
	return result;
}

static void
_tuner_window___lambda89_ (TunerWindow* self)
{
	tuner_window_check_online_status (self);
}

static void
__tuner_window___lambda89__g_object_notify (GObject* _sender,
                                            GParamSpec* pspec,
                                            gpointer self)
{
	_tuner_window___lambda89_ ((TunerWindow*) self);
}

static GObject *
tuner_window_constructor (GType type,
                          guint n_construct_properties,
                          GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	TunerWindow * self;
	GdkGeometry _tmp0_ = {0};
	TunerSettings* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	GVariant* _tmp4_;
	TunerSettings* _tmp5_;
	gboolean _tmp6_;
	gboolean _tmp7_;
	GVariant* _tmp8_;
	TunerSettings* _tmp9_;
	gboolean _tmp10_;
	gboolean _tmp11_;
	GVariant* _tmp12_;
	TunerSettings* _tmp13_;
	gboolean _tmp14_;
	gboolean _tmp15_;
	GVariant* _tmp16_;
	TunerSettings* _tmp17_;
	gboolean _tmp18_;
	gboolean _tmp19_;
	GVariant* _tmp20_;
	TunerApplication* _tmp21_;
	TunerApplication* _tmp22_;
	parent_class = G_OBJECT_CLASS (tuner_window_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TUNER_TYPE_WINDOW, TunerWindow);
	gtk_window_set_icon_name ((GtkWindow*) self, TUNER_APPLICATION_APP_ID);
	g_action_map_add_action_entries ((GActionMap*) self, TUNER_WINDOW_ACTION_ENTRIES, (gint) G_N_ELEMENTS (TUNER_WINDOW_ACTION_ENTRIES), self);
	gtk_window_set_title ((GtkWindow*) self, TUNER_WINDOW_WINDOW_NAME);
	g_object_set ((GtkWindow*) self, "window-position", GTK_WIN_POS_CENTER, NULL);
	memset (&_tmp0_, 0, sizeof (GdkGeometry));
	_tmp0_.min_height = TUNER_WINDOW_GEOMETRY_MIN_HEIGHT;
	_tmp0_.min_width = TUNER_WINDOW_GEOMETRY_MIN_WIDTH;
	gtk_window_set_geometry_hints ((GtkWindow*) self, NULL, &_tmp0_, GDK_HINT_MIN_SIZE);
	_tmp1_ = self->priv->_settings;
	_tmp2_ = tuner_settings_get_do_not_vote (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = _variant_new11 (_tmp3_);
	g_action_group_change_action_state ((GActionGroup*) self, TUNER_WINDOW_ACTION_DISABLE_TRACKING, _tmp4_);
	_g_variant_unref0 (_tmp4_);
	_tmp5_ = self->priv->_settings;
	_tmp6_ = tuner_settings_get_auto_play (_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = _variant_new12 (_tmp7_);
	g_action_group_change_action_state ((GActionGroup*) self, TUNER_WINDOW_ACTION_ENABLE_AUTOPLAY, _tmp8_);
	_g_variant_unref0 (_tmp8_);
	_tmp9_ = self->priv->_settings;
	_tmp10_ = tuner_settings_get_start_on_starred (_tmp9_);
	_tmp11_ = _tmp10_;
	_tmp12_ = _variant_new13 (_tmp11_);
	g_action_group_change_action_state ((GActionGroup*) self, TUNER_WINDOW_ACTION_START_ON_STARRED, _tmp12_);
	_g_variant_unref0 (_tmp12_);
	_tmp13_ = self->priv->_settings;
	_tmp14_ = tuner_settings_get_stream_info (_tmp13_);
	_tmp15_ = _tmp14_;
	_tmp16_ = _variant_new14 (_tmp15_);
	g_action_group_change_action_state ((GActionGroup*) self, TUNER_WINDOW_ACTION_STREAM_INFO, _tmp16_);
	_g_variant_unref0 (_tmp16_);
	_tmp17_ = self->priv->_settings;
	_tmp18_ = tuner_settings_get_stream_info_fast (_tmp17_);
	_tmp19_ = _tmp18_;
	_tmp20_ = _variant_new15 (_tmp19_);
	g_action_group_change_action_state ((GActionGroup*) self, TUNER_WINDOW_ACTION_STREAM_INFO_FAST, _tmp20_);
	_g_variant_unref0 (_tmp20_);
	g_signal_connect_object ((GtkWidget*) self, "delete-event", (GCallback) __tuner_window___lambda88__gtk_widget_delete_event, self, 0);
	_tmp21_ = tuner_app ();
	_tmp22_ = _tmp21_;
	g_signal_connect_object ((GObject*) _tmp22_, "notify::is-online", (GCallback) __tuner_window___lambda89__g_object_notify, self, 0);
	_g_object_unref0 (_tmp22_);
	return obj;
}

static void
tuner_window_class_init (TunerWindowClass * klass,
                         gpointer klass_data)
{
	tuner_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &TunerWindow_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_tuner_window_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_tuner_window_set_property;
	G_OBJECT_CLASS (klass)->constructor = tuner_window_constructor;
	G_OBJECT_CLASS (klass)->finalize = tuner_window_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNER_WINDOW_SETTINGS_PROPERTY, tuner_window_properties[TUNER_WINDOW_SETTINGS_PROPERTY] = g_param_spec_object ("settings", "settings", "settings", TUNER_TYPE_SETTINGS, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNER_WINDOW_PLAYER_CTRL_PROPERTY, tuner_window_properties[TUNER_WINDOW_PLAYER_CTRL_PROPERTY] = g_param_spec_object ("player-ctrl", "player-ctrl", "player-ctrl", TUNER_TYPE_PLAYER_CONTROLLER, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNER_WINDOW_DIRECTORY_PROPERTY, tuner_window_properties[TUNER_WINDOW_DIRECTORY_PROPERTY] = g_param_spec_object ("directory", "directory", "directory", TUNER_TYPE_DIRECTORY_CONTROLLER, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNER_WINDOW_ACTIVE_PROPERTY, tuner_window_properties[TUNER_WINDOW_ACTIVE_PROPERTY] = g_param_spec_boolean ("active", "active", "active", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNER_WINDOW_WIDTH_PROPERTY, tuner_window_properties[TUNER_WINDOW_WIDTH_PROPERTY] = g_param_spec_int ("width", "width", "width", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNER_WINDOW_HEIGHT_PROPERTY, tuner_window_properties[TUNER_WINDOW_HEIGHT_PROPERTY] = g_param_spec_int ("height", "height", "height", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	tuner_window_signals[TUNER_WINDOW_REFRESH_SAVED_SEARCHES_SIG_SIGNAL] = g_signal_new ("refresh-saved-searches-sig", TUNER_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__BOOLEAN_STRING, G_TYPE_NONE, 2, G_TYPE_BOOLEAN, G_TYPE_STRING);
}

static void
tuner_window_instance_init (TunerWindow * self,
                            gpointer klass)
{
	self->priv = tuner_window_get_instance_private (self);
	self->priv->_start_on_starred = FALSE;
}

static void
tuner_window_finalize (GObject * obj)
{
	TunerWindow * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TUNER_TYPE_WINDOW, TunerWindow);
	_g_object_unref0 (self->priv->_settings);
	_g_object_unref0 (self->priv->_player_ctrl);
	_g_object_unref0 (self->priv->_directory);
	_g_object_unref0 (self->priv->_headerbar);
	_g_object_unref0 (self->priv->_display);
	G_OBJECT_CLASS (tuner_window_parent_class)->finalize (obj);
}

/**
 * The main application window for the Tuner app.
 * 
 * This class extends Gtk.ApplicationWindow and serves as the primary container
 * for all other widgets and functionality in the Tuner application.
 */
static GType
tuner_window_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (TunerWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuner_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TunerWindow), 0, (GInstanceInitFunc) tuner_window_instance_init, NULL };
	GType tuner_window_type_id;
	tuner_window_type_id = g_type_register_static (gtk_application_window_get_type (), "TunerWindow", &g_define_type_info, 0);
	TunerWindow_private_offset = g_type_add_instance_private (tuner_window_type_id, sizeof (TunerWindowPrivate));
	return tuner_window_type_id;
}

GType
tuner_window_get_type (void)
{
	static volatile gsize tuner_window_type_id__once = 0;
	if (g_once_init_enter (&tuner_window_type_id__once)) {
		GType tuner_window_type_id;
		tuner_window_type_id = tuner_window_get_type_once ();
		g_once_init_leave (&tuner_window_type_id__once, tuner_window_type_id);
	}
	return tuner_window_type_id__once;
}

static void
_vala_tuner_window_get_property (GObject * object,
                                 guint property_id,
                                 GValue * value,
                                 GParamSpec * pspec)
{
	TunerWindow * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TUNER_TYPE_WINDOW, TunerWindow);
	switch (property_id) {
		case TUNER_WINDOW_SETTINGS_PROPERTY:
		g_value_set_object (value, tuner_window_get_settings (self));
		break;
		case TUNER_WINDOW_PLAYER_CTRL_PROPERTY:
		g_value_set_object (value, tuner_window_get_player_ctrl (self));
		break;
		case TUNER_WINDOW_DIRECTORY_PROPERTY:
		g_value_set_object (value, tuner_window_get_directory (self));
		break;
		case TUNER_WINDOW_ACTIVE_PROPERTY:
		g_value_set_boolean (value, tuner_window_get_active (self));
		break;
		case TUNER_WINDOW_WIDTH_PROPERTY:
		g_value_set_int (value, tuner_window_get_width (self));
		break;
		case TUNER_WINDOW_HEIGHT_PROPERTY:
		g_value_set_int (value, tuner_window_get_height (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_tuner_window_set_property (GObject * object,
                                 guint property_id,
                                 const GValue * value,
                                 GParamSpec * pspec)
{
	TunerWindow * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TUNER_TYPE_WINDOW, TunerWindow);
	switch (property_id) {
		case TUNER_WINDOW_SETTINGS_PROPERTY:
		tuner_window_set_settings (self, g_value_get_object (value));
		break;
		case TUNER_WINDOW_PLAYER_CTRL_PROPERTY:
		tuner_window_set_player_ctrl (self, g_value_get_object (value));
		break;
		case TUNER_WINDOW_DIRECTORY_PROPERTY:
		tuner_window_set_directory (self, g_value_get_object (value));
		break;
		case TUNER_WINDOW_ACTIVE_PROPERTY:
		tuner_window_set_active (self, g_value_get_boolean (value));
		break;
		case TUNER_WINDOW_WIDTH_PROPERTY:
		tuner_window_set_width (self, g_value_get_int (value));
		break;
		case TUNER_WINDOW_HEIGHT_PROPERTY:
		tuner_window_set_height (self, g_value_get_int (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_array_destroy (gpointer array,
                     gssize array_length,
                     GDestroyNotify destroy_func)
{
	if ((array != NULL) && (destroy_func != NULL)) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}

static void
_vala_array_free (gpointer array,
                  gssize array_length,
                  GDestroyNotify destroy_func)
{
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

